import { RcsbFv3DCustomInterface} from "../RcsbFv3D/RcsbFv3DCustom";
import { DebugUtil } from "./tmdet-extension/debug-utils";
import { createFeatureViewerConfing } from "./FeatureViewConfig";
import { fetchDescriptor, registerRegionDescriptorData } from "./UniTmpHelper";
import { TmFv3DCustom } from "./tmdet-viewer/TmFv3DCustom";
import { RcsbFv, RcsbFvRowConfigInterface } from "@rcsb/rcsb-saguaro";
import { getStyleColorString, updateSiteColors } from "./tmdet-extension/tmdet-color-theme";

function createViewer(configParams: any) {
    document.addEventListener("DOMContentLoaded", (event) => {
        (async function(event) {

            DebugUtil.enableLog();

            const panel3dConfig = await createConfig(configParams);
            DebugUtil.log('Panel config', JSON.stringify(panel3dConfig));

            const panel3d = new TmFv3DCustom(panel3dConfig);
            panel3d.render();

        })();
    });
}

async function createConfig(configParams: any): Promise<RcsbFv3DCustomInterface> {
    const pdbEntry = configParams.pdbEntry;
    const labelAsymId = configParams.labelAsymId;
    const params: any = {
        title: configParams.title,
        side1: configParams.side1,
        htpEntry: configParams.htpEntry,
        pdbEntry: configParams.pdbEntry,
        labelAsymId: labelAsymId,
        pdbtmEntry: `${pdbEntry}_${labelAsymId}`
    };

    if (pdbEntry) {
        const descriptorUrl = `/${pdbEntry}.json`;
        registerRegionDescriptorData(descriptorUrl, params.side1);
    }
    if (configParams.configData) {
        params.fvConfigData = await fetchDescriptor(configParams.configData);
    }

    const { sequenceConfig, molstarConfig } = await createFeatureViewerConfing(params);
    const panel3dConfig: RcsbFv3DCustomInterface = {
        elementId: configParams.elementId,
        sequencePanelConfig: sequenceConfig,
        structurePanelConfig: molstarConfig
    };
    molstarConfig.structureViewerConfig.viewerProps.layoutShowSequence = true;
    (molstarConfig as any).loadConfig.loadParams.params = {
        preset: { modelIndex: 0 }
    }

    return panel3dConfig;
}

/**
 * Create an Rcsb Saguaro 1D viewer instance.
 *
 * @param elementId id of parent element
 * @param url source of JSON config
 * @param side1 side1 paramter to update site colors, if it is needed
 */
async function createRcsbFeatureViewer(params: {
    elementId: string, url: string, side1: string, trackWidth?: number, rowTitleWidth?: number }) {

    updateSiteColors(params.side1 as any);

    const featureTracks = await fetchDescriptor(params.url);
    const blockConfig = featureTracks.blockConfig;
    const boardConfig = blockConfig[0].featureViewConfig[0].boardConfig;
    boardConfig.trackWidth = params.trackWidth ?? 600;
    boardConfig.rowTitleWidth = params.rowTitleWidth ?? 160;

    const rowConfig = blockConfig[0].featureViewConfig[0].rowConfig;
    rowConfig.forEach((track: RcsbFvRowConfigInterface) => {
        track.displayConfig?.forEach(region => {
            const css = (region as any).css;
            if (css) {
                region.displayColor = getStyleColorString(css);
            }
        });
    });

    const pfv = new RcsbFv({
        boardConfigData: boardConfig,
        rowConfigData: rowConfig,
        elementId: params.elementId
    });
}

export { DebugUtil, createViewer, createRcsbFeatureViewer };
